## THIS IS AN AWK PROGRAM FOR PREPARING 1--1 TCX TABLES FROM SOURCE ENCODINGS
## Only codes > 127 are remapped; for not encoded characters
## the identical mapping is used.

BEGIN {
# READING DATA FILE:
  while (getline < DAT > 0)
    if ($2==">>") file_name[$1]=$3
    else if ($0!="") prolog[++prolog[0]]=$0
# READING ENCODING FILES:
  max_length=0
  while (getline < file_name["S"] > 0) 
    if (($1~/[0-9]+/) && ($1>127) && (length($2)==1) && ($3!="")) {
      if ($1 in source_name) print "REPEATED SOURCE CODE", $3, $1 > "CON"
      if ($3 in source_code) print "REPEATED SOURCE NAME", $3, $1 > "CON"
      source_code[$3]=$1; source_name[$1]=$3
      if (length($3)>max_length) max_length=length($3)
    }
  while (getline < file_name["T"] > 0) 
    if (($1~/[0-9]+/) && ($1>127) && ($2!="")) {
      if ($2!="@") {# @ denotes a dummy name in target encoding tables
        if ($1 in target_name) print "REPEATED TARGET CODE", $2, $1 > "CON"
        if ($2 in target_code) print "REPEATED TARGET NAME", $2, $1 > "CON"
      }
      target_code[$2]=$1; target_name[$1]=$2
      if (length($2)>max_length) max_length=length($2)
    }
# CROSS-REFERENCING:
  for (i in target_code) if (i in source_code) {
    encode[source_code[i]]=target_code[i]
    decode[target_code[i]]=source_code[i]
  }
# CLOSING PERMUTATION CYCLES:
  for (i=128; i<=255; ++i)
    if (!(i in used) && !(i in decode)) close_cycle(i)

# PRINTING RESULTS:
  for (i=1; i<=prolog[0]; ++i) print_string(prolog[i])
  print_string("\n%% MAIN ENCODING TABLE:")
  for (i=128; i<=255; ++i) if (i in encode) print_encoding_item(i)
  print_string("\n%% ARTIFICIAL CLOSURES (MAKE MAPPING A PERMUTATION):")
  for (i=128; i<=255; ++i) if (i in closure) print_closure(i)
  print_string("\n%% ODD SINGLETS:")
  for (i=128; i<=255; ++i) if (i in singlet) print_odd_singlet(i)
#  print_string("\n%% LOW CODE SINGLETS:")
#  for (i=0; i<32; ++i) print_lower_singlet(i)
}

function close_cycle(i, j) {
  j=i;
  while (j in encode) {
    if (j in used) {print "SHOULDN'T HAPPEN", i, j > "CON"; exit}
    used[j]++; j=encode[j]
  }
  if (i==j) singlet[j]=i
  else closure[j]=i
}

function print_string(s) {print s > file_name["R"]}
function print_encoding_item(i) {
  print hex(i), hex(encode[i]), "%", dec(i), dec(encode[i]),
        source_name[i] > file_name["R"]
}
function print_closure(i, j,k) {
  print "" > file_name["R"]
  k=closure[i]
  do {
    j=k; k=encode[j]
    print "%" , dec(j), dec(k), source_name[j] > file_name["R"]
  } while (k!=i)
  print hex(i), hex(closure[i]), "%",
        dec(i), "-->", dec(closure[i]) > file_name["R"]
}
function print_odd_singlet(i) {
  print hex(i), hex(i), "%", dec(i),
        sprintf("%" (max_length+1) "s", source_name[i]),
        sprintf("%" (max_length+1) "s", target_name[i]) > file_name["R"]
}
function print_lower_singlet(i) {
  print hex(i), hex(i), "%", dec(i) > file_name["R"]
}
function hex(i) {return sprintf("0x%02x",i)}
function dec(i) {return sprintf("%3d",i)}

